/// <reference types="node" />
import OS from "os";
import { Credentials } from "@aws-sdk/types";
import type { Client } from "./configuration.js";
/***
 * Client Credentials
 * ---
 *
 * Creates a credential provider function that reads from a shared credentials file at ~/.aws/credentials and a shared
 * configuration file at ~/.aws/config.
 *
 * Both files are expected to be INI formatted with section names corresponding to
 * profiles.
 *
 * Sections in the credentials file are treated as profile names, whereas profile sections in the config file
 * must have the format of[profile profile-name], except for the default profile.
 *
 * @example
 * import { Credentials } from "credentials";
 * const credentials = await Credentials.initialize();
 * console.log(credentials);
 *
 * @example
 * /// An Example, Abstract Implementation
 * import { Credential } from "@cloud-technology/aws-credential";
 *
 * class Client extends Credential {
 *     protected constructor( profile: string = "default" ) {
 *         super( profile );
 *     }
 *
 *     public static async create( debug: boolean = false ) {
 *         return await ( new Client() ).initialize( debug );
 *     }
 *
 *     private async initialize( debug: boolean = false ) {
 *         await this.hydrate();
 *
 *         /// Any AWS Related SDK-V3 Client
 *         this.service = new SDK-V3-Client( {
 *             tls: true,
 *             sha256: Sha256,
 *             credentials: this.settings,
 *             customUserAgent: "Cloud-Technology-API",
 *             region: Process.env?.["AWS_DEFAULT_REGION"] ?? "us-east-2"
 *         } );
 *
 *         ( debug ) && console.debug( "[Debug] Client Instance" + ":", Utility.inspect( this, { showHidden: true, depth: Infinity } ) );
 *
 *         return this;
 *     }
 * }
 *
 * export { Client };
 * export default { Client };
 *
 */
declare class Credential {
    private static $;
    readonly profile: string;
    /***
     * Returns information about the currently effective user. On POSIX platforms, this is typically a subset of the
     * password file. The returned object includes the username, uid, gid, shell, and homedir. On Windows, the uid
     * and gid fields are -1, and shell is null. The value of homedir returned by os.userInfo() is provided by the
     * operating system. This differs from the result of os.homedir(), which queries environment variables for the home
     * directory before falling back to the operating system response.
     *
     * Throws a SystemError if a user has no username or homedir.
     *
     */
    readonly user: OS.UserInfo<string>;
    settings: Promise<Credentials>;
    service?: Client<any, any, any>;
    protected id?: Function | null;
    protected key?: Function | null;
    protected region?: Function | null;
    private environment;
    protected constructor(profile: string);
    static initialize(profile?: string): Promise<Credential>;
    /***
     * Initialize Client via Environment or Profile Credentials
     *
     * @returns {Promise<void>}
     *
     */
    protected hydrate(): Promise<this>;
}
export { Credential };
declare const _default: {
    Credential: typeof Credential;
};
export default _default;
//# sourceMappingURL=credential.d.ts.map