"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Credential = void 0;
const os_1 = __importDefault(require("os"));
const process_1 = __importDefault(require("process"));
const credential_provider_node_1 = require("@aws-sdk/credential-provider-node");
/***
 * Client Credentials
 * ---
 *
 * Creates a credential provider function that reads from a shared credentials file at ~/.aws/credentials and a shared
 * configuration file at ~/.aws/config.
 *
 * Both files are expected to be INI formatted with section names corresponding to
 * profiles.
 *
 * Sections in the credentials file are treated as profile names, whereas profile sections in the config file
 * must have the format of[profile profile-name], except for the default profile.
 *
 * @example
 * import { Credentials } from "credentials";
 * const credentials = await Credentials.initialize();
 * console.log(credentials);
 *
 * @example
 * /// An Example, Abstract Implementation
 * import { Credential } from "@cloud-technology/aws-credential";
 *
 * class Client extends Credential {
 *     protected constructor( profile: string = "default" ) {
 *         super( profile );
 *     }
 *
 *     public static async create( debug: boolean = false ) {
 *         return await ( new Client() ).initialize( debug );
 *     }
 *
 *     private async initialize( debug: boolean = false ) {
 *         await this.hydrate();
 *
 *         /// Any AWS Related SDK-V3 Client
 *         this.service = new SDK-V3-Client( {
 *             tls: true,
 *             sha256: Sha256,
 *             credentials: this.settings,
 *             customUserAgent: "Cloud-Technology-API",
 *             region: Process.env?.["AWS_DEFAULT_REGION"] ?? "us-east-2"
 *         } );
 *
 *         ( debug ) && console.debug( "[Debug] Client Instance" + ":", Utility.inspect( this, { showHidden: true, depth: Infinity } ) );
 *
 *         return this;
 *     }
 * }
 *
 * export { Client };
 * export default { Client };
 *
 */
class Credential {
    constructor(profile) {
        /***
         * Returns information about the currently effective user. On POSIX platforms, this is typically a subset of the
         * password file. The returned object includes the username, uid, gid, shell, and homedir. On Windows, the uid
         * and gid fields are -1, and shell is null. The value of homedir returned by os.userInfo() is provided by the
         * operating system. This differs from the result of os.homedir(), which queries environment variables for the home
         * directory before falling back to the operating system response.
         *
         * Throws a SystemError if a user has no username or homedir.
         *
         */
        this.user = os_1.default.userInfo();
        this.id = () => { var _a; return (_a = process_1.default.env["AWS_ACCESS_KEY_ID"]) !== null && _a !== void 0 ? _a : null; };
        this.key = () => { var _a; return (_a = process_1.default.env["AWS_SECRET_ACCESS_TOKEN"]) !== null && _a !== void 0 ? _a : null; };
        this.region = () => { var _a; return (_a = process_1.default.env["AWS_DEFAULT_REGION"]) !== null && _a !== void 0 ? _a : null; };
        this.profile = profile;
        this.settings = (0, credential_provider_node_1.defaultProvider)({
            profile: this.profile
        })();
    }
    static initialize(profile = "default") {
        return __awaiter(this, void 0, void 0, function* () {
            const instance = new Credential(profile);
            yield instance.hydrate();
            return instance;
        });
    }
    /***
     * Initialize Client via Environment or Profile Credentials
     *
     * @returns {Promise<void>}
     *
     */
    hydrate() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.settings;
            return this;
        });
    }
}
exports.Credential = Credential;
exports.default = { Credential };
