import OS from "os";
import Process from "process";

import { defaultProvider } from "@aws-sdk/credential-provider-node";

import { Credentials } from "@aws-sdk/types";

import type { LambdaClient as Client } from "@aws-sdk/client-lambda";

/***
 * Client Credentials
 * ---
 *
 * Creates a credential provider function that reads from a shared credentials file at ~/.aws/credentials and a shared
 * configuration file at ~/.aws/config.
 *
 * Both files are expected to be INI formatted with section names corresponding to
 * profiles.
 *
 * Sections in the credentials file are treated as profile names, whereas profile sections in the config file
 * must have the format of[profile profile-name], except for the default profile.
 *
 * @example
 * import { Credentials } from "credentials";
 * const credentials = await Credentials.initialize();
 * console.log(credentials);
 *
 * @example
 * /// An Example, Abstract Implementation
 * import { Credential } from "@cloud-technology/aws-credential";
 *
 * class Client extends Credential {
 *     protected constructor( profile: string = "default" ) {
 *         super( profile );
 *     }
 *
 *     public static async create( debug: boolean = false ) {
 *         return await ( new Client() ).initialize( debug );
 *     }
 *
 *     private async initialize( debug: boolean = false ) {
 *         await this.hydrate();
 *
 *         /// Any AWS Related SDK-V3 Client
 *         this.service = new SDK-V3-Client( {
 *             tls: true,
 *             sha256: Sha256,
 *             credentials: this.settings,
 *             customUserAgent: "Cloud-Technology-API",
 *             region: Process.env?.["AWS_DEFAULT_REGION"] ?? "us-east-2"
 *         } );
 *
 *         ( debug ) && console.debug( "[Debug] Client Instance" + ":", Utility.inspect( this, { showHidden: true, depth: Infinity } ) );
 *
 *         return this;
 *     }
 * }
 *
 * export { Client };
 * export default { Client };
 *
 */
class Credential {
    public readonly profile: string;

    /***
     * Returns information about the currently effective user. On POSIX platforms, this is typically a subset of the
     * password file. The returned object includes the username, uid, gid, shell, and homedir. On Windows, the uid
     * and gid fields are -1, and shell is null. The value of homedir returned by os.userInfo() is provided by the
     * operating system. This differs from the result of os.homedir(), which queries environment variables for the home
     * directory before falling back to the operating system response.
     *
     * Throws a SystemError if a user has no username or homedir.
     *
     */
    public readonly user = OS.userInfo();

    public settings: Promise<Credentials>;

    public service?: Client;

    protected constructor(profile: string) {
        this.profile = profile;

        this.settings = defaultProvider( {
            profile: this.profile
        } )();
    }

    public static async initialize(profile: string = "default") {
        const instance = new Credential( profile );

        await instance.hydrate();

        return instance;
    }

    protected id?: Function | null = () => Process.env[ "AWS_ACCESS_KEY_ID" ] ?? null;

    protected key?: Function | null = () => Process.env[ "AWS_SECRET_ACCESS_TOKEN" ] ?? null;

    protected region?: Function | null = () => Process.env[ "AWS_DEFAULT_REGION" ] ?? null;

    /***
     * Initialize Client via Environment or Profile Credentials
     *
     * @returns {Promise<void>}
     *
     */

    protected async hydrate(): Promise<this> {
        await this.settings;

        return this;
    }
}

export { Credential };

export default { Credential };
